/*
	Data Structure Collection
	 by x86 Assembly Language

	Programmed by Woonsan Yeo, Paradoxoft Corporation.
*/

#include "stdafx.h"

void StackTest();
void LinkedListTest();

// ----------------------------------- Stack Section -----------------------------------
struct StackRegistrationStructure;

struct StackRegistrationStructure
{
	unsigned int CurrentStackPoint; // Current Stack Point
	void* AllocatedSpace; // Address of Allocated Space for Stack
	unsigned int AllocatedSize; // Size of Allocated Space for Stack
};

extern "C" void __stdcall RegisterStack(StackRegistrationStructure* StructureAddress, unsigned int PreAllocationSize);
extern "C" void __stdcall ReleaseStack(StackRegistrationStructure* StructureAddress);
extern "C" void __stdcall CleanupStack(StackRegistrationStructure* StructureAddress);
extern "C" void __stdcall Push8(StackRegistrationStructure* StructureAddress, char DataToPush);
extern "C" void __stdcall Push16(StackRegistrationStructure* StructureAddress, short DataToPush);
extern "C" void __stdcall Push32(StackRegistrationStructure* StructureAddress, int DataToPush);
extern "C" char __stdcall Pop8(StackRegistrationStructure* StructureAddress);
extern "C" short __stdcall Pop16(StackRegistrationStructure* StructureAddress);
extern "C" int __stdcall Pop32(StackRegistrationStructure* StructureAddress);

void StackTest()
{
	printf("< Stack Section >\n");

	printf("Register stack...\t\t");
	StackRegistrationStructure teststack;
	RegisterStack(&teststack, 4194304); // Pre-Allocate 4kb
	printf("ok\n");

	printf("Push 8-bit and Pop\n");
	Push8(&teststack, 21);
	printf(" > %d\n", Pop8(&teststack));
	printf("Push 16-bit and Pop\n");
	Push16(&teststack, 15715);
	printf(" > %d\n", Pop16(&teststack));
	printf("Push 32-bit and Pop\n");
	Push32(&teststack, 4845126);
	printf(" > %d\n", Pop32(&teststack));

	printf("Release stack...\t\t");
	ReleaseStack(&teststack);
	printf("ok\n");
}

// ----------------------------------- Linked-List Section -----------------------------------
struct LinkedListRegistrationStructure;
struct LinkedListEntryStructure;

struct LinkedListRegistrationStructure
{
	LinkedListEntryStructure* FirstEntry;
	LinkedListEntryStructure* LastEntry;
	unsigned int TotalEntryCount;
};

struct LinkedListEntryStructure
{
	unsigned int ID;
	void* DataPoint;
	LinkedListEntryStructure* PreviousEntry;
	LinkedListEntryStructure* NextEntry;
};

extern "C" void __stdcall RegisterLinkedList(LinkedListRegistrationStructure* StructureAddress);
extern "C" void __stdcall ReleaseLinkedList(LinkedListRegistrationStructure* StructureAddress);
extern "C" void __stdcall AddEntry(LinkedListRegistrationStructure* StructureAddress, unsigned int ID, char* DataPoint);
//extern "C" void __stdcall InsertEntryNext(LinkedListRegistrationStructure* StructureAddress, unsigned int ID, void* DataPoint, LinkedListEntryStructure* MainEntry);
//extern "C" void __stdcall InsertEntryPrevious(LinkedListRegistrationStructure* StructureAddress, unsigned int ID, void* DataPoint, LinkedListEntryStructure* MainEntry);
extern "C" void __stdcall DeleteEntry(LinkedListRegistrationStructure* StructureAddress, LinkedListEntryStructure* RunEntry);
extern "C" LinkedListEntryStructure* __stdcall SearchEntry(LinkedListRegistrationStructure* StructureAddress, unsigned int ID);

static char testvariable_1[] = "THIS IS A TEST VARIABLE THAT POINTED BY THE FIRST ENTRY.";
static char testvariable_2[] = "THIS IS A TEST VARIABLE THAT POINTED BY THE SECOND ENTRY.";
static char testvariable_3[] = "THIS IS A TEST VARIABLE THAT POINTED BY THE THIRD ENTRY.";
static char testvariable_4[] = "THIS IS A TEST VARIABLE THAT POINTED BY THE FOURTH ENTRY.";
static char testvariable_5[] = "THIS IS A TEST VARIABLE THAT POINTED BY THE FIFTH ENTRY.";

void LinkedListTest()
{
	printf("< Linked-List Section >\n");

	printf("Register linked-list...\t\t");
	LinkedListRegistrationStructure testlinkedlist;
	RegisterLinkedList(&testlinkedlist);
	printf("ok\n");

	printf("Create an entry in Linked-List\n");
	AddEntry(&testlinkedlist, 1, testvariable_1);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Create an entry in Linked-List\n");
	AddEntry(&testlinkedlist, 2, testvariable_2);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Create an entry in Linked-List\n");
	AddEntry(&testlinkedlist, 3, testvariable_3);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Create an entry in Linked-List\n");
	AddEntry(&testlinkedlist, 4, testvariable_4);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Create an entry in Linked-List\n");
	AddEntry(&testlinkedlist, 5, testvariable_4);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Search an entry, ID - 2 in Linked-List\n");
	LinkedListEntryStructure* PData_2 = SearchEntry(&testlinkedlist, 3);
	if ((int)PData_2 == -1) // If not found
	{
		printf(" > Not found!\n\n");
	}
	else
	{
		LinkedListEntryStructure SStc_2 = *PData_2;
		printf(" > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", SStc_2.ID, SStc_2.DataPoint, SStc_2.DataPoint, SStc_2.PreviousEntry, SStc_2.NextEntry);
	}

	printf("Delete the first entry in Linked-List\n");
	DeleteEntry(&testlinkedlist, testlinkedlist.FirstEntry);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).NextEntry);
	
	printf("Delete the first entry in Linked-List\n");
	DeleteEntry(&testlinkedlist, testlinkedlist.FirstEntry);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.FirstEntry).NextEntry);

	printf("Delete the last entry in Linked-List\n");
	DeleteEntry(&testlinkedlist, testlinkedlist.LastEntry);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Delete the last entry in Linked-List\n");
	DeleteEntry(&testlinkedlist, testlinkedlist.LastEntry);
	printf(" > FirstEntry: %x, LastEntry: %x, TotalEntryCount: %d\n > ID: %d, DataPoint: %x(%s), PreviousEntry: %x, NextEntry: %x\n\n", testlinkedlist.FirstEntry, testlinkedlist.LastEntry, testlinkedlist.TotalEntryCount, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).ID, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).DataPoint, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).PreviousEntry, ((LinkedListEntryStructure)*testlinkedlist.LastEntry).NextEntry);

	printf("Release linked-list...\t\t");
	ReleaseLinkedList(&testlinkedlist);
	printf("ok\n");
	}

// ----------------------------------- Main -----------------------------------
int _tmain(int argc, _TCHAR* argv[])
{
	printf("Data Structure by x86 Assembly\n Programmed by Woonsan Yeo, Paradoxoft Corporation.\n\n");
	StackTest();
	printf("\n");
	LinkedListTest();

	_getch();
	return 0;
}