comment ~
	Data Structure Collection
	 by x86 Assembly Language
	
	Stack Section
	
	Programmed by Woonsan Yeo, Paradoxoft Corporation.
~

.686
.mmx
.xmm
.model flat, stdcall

GetProcessHeap		proto
HeapAlloc			proto	hHeap:DWORD, dwFlags:DWORD, dwBytes:DWORD
HeapFree			proto	hHeap:DWORD, dwFlags:DWORD, lpMem:DWORD

StackRegistrationStructure		struct
	CurrentStackPoint	DWORD	?															; Current Stack Point
	AllocatedSpace		DWORD	?															; Address of Allocated Space for Stack
	AllocatedSize		DWORD	?															; Size of Allocated Space for Stack
StackRegistrationStructure		ends

.code
	; Management Operation
	comment !
		Stack Registration - Allocate stack space and set structure information
		 Name		/	RegisterStack
		 Parameters /	StructureAddress - Address of Stack Registration Structure
						PreAllocationSize - Size of Stack Space Pre-Allocation !
	RegisterStack		proc	StructureAddress:DWORD, PreAllocationSize:DWORD
		pushad
		invoke GetProcessHeap
		invoke HeapAlloc, eax, 0, dword ptr [PreAllocationSize]								; Allocate space for stack
		mov ebx, dword ptr [StructureAddress]
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace], eax			; Set [AllocatedSpace]
		mov eax, dword ptr [PreAllocationSize]
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSize], eax			; Set [AllocatedSize]
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], eax		; Set [CurrentStackPoint]
		popad
		ret
	RegisterStack		endp
	
	comment!
		Stack Release - Release stack space and clear structure information
		Name		/	ReleaseStack
		Parameters	/	StructureAddress - Address of Stack Registration Structure !
	ReleaseStack		proc	StructureAddress:DWORD
		pushad
		invoke GetProcessHeap
		mov ecx, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [ecx]).AllocatedSpace]
		invoke HeapFree, eax, 0, ebx
		mov edi, dword ptr [StructureAddress]
		xor eax, eax
		mov ecx, 3																			; Structure is 3 * 4 bytes
		cld
		rep stosd
		popad
		ret
	ReleaseStack		endp
	
	comment!
		Clean Up Stack - Clean up stack space and set pointer to 0
		Name		/	CleanupStack
		Parameters	/	StructureAddress - Address of Stack Registration Structure !
	CleanupStack		proc	StructureAddress:DWORD
		pushad
		mov eax, dword ptr [StructureAddress]
		mov edi, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]				; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSize]				; Get [AllocatedSize]
		
		mov ecx, edx
		shr ecx, 4																			; Calculate quotient
		xorps xmm0, xmm0																	; Generate zero 128-bit register
		
		by_oword:
			movdqu oword ptr [edi], xmm0													; Put zero
			add edi, 8																		; Process 8-byte in a time
			dec ecx
			or ecx, ecx
			jz by_oword_end																	; End processing
		jmp by_oword
		by_oword_end:
		
		xor eax, eax
		mov ecx, edx
		and ecx, 15																			; Calculate remainder
		cld
		rep stosb
		popad
		ret
	CleanupStack		endp
	
	; Data Operation
	Push8				proc	StructureAddress:DWORD, DataToPush:BYTE
		push ebx
		push ecx
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]			; Get [AllocatedSpace]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint]		; Get [CurrentStackPoint]
		sub ecx, 1
		mov dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint], ecx		; Set [CurrentStackPoint]
		mov al, byte ptr [DataToPush]
		mov byte ptr [ebx + ecx], al														; Push value into the stack
		pop ecx
		pop ebx
		ret
	Push8				endp
	
	Push16				proc	StructureAddress:DWORD, DataToPush:WORD
		push ebx
		push ecx
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]			; Get [AllocatedSpace]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint]		; Get [CurrentStackPoint]
		sub ecx, 2
		mov dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint], ecx		; Set [CurrentStackPoint]
		mov ax, word ptr [DataToPush]
		mov word ptr [ebx + ecx], ax														; Push value into the stack
		pop ecx
		pop ebx
		ret
	Push16				endp
	
	Push32				proc	StructureAddress:DWORD, DataToPush:DWORD
		push ebx
		push ecx
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]			; Get [AllocatedSpace]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint]		; Get [CurrentStackPoint]
		sub ecx, 4
		mov dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint], ecx		; Set [CurrentStackPoint]
		mov eax, dword ptr [DataToPush]
		mov dword ptr [ebx + ecx], eax														; Push value into the stack
		pop ecx
		pop ebx
		ret
	Push32				endp
	
	Push64				proc	StructureAddress:DWORD, DataToPush:QWORD					; Not supported in C
		push ebx
		push ecx
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]			; Get [AllocatedSpace]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint]		; Get [CurrentStackPoint]
		sub ecx, 8
		mov dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint], ecx		; Set [CurrentStackPoint]
		movq mm0, qword ptr [DataToPush]
		movq qword ptr [ebx + ecx], mm0														; Push value into the stack
		pop ecx
		pop ebx
		ret
	Push64				endp
	
	Push128				proc	StructureAddress:DWORD, DataToPush:OWORD					; Not supported in C
		push ebx
		push ecx
		mov eax, dword ptr [StructureAddress]
		mov ebx, dword ptr [(StackRegistrationStructure ptr [eax]).AllocatedSpace]			; Get [AllocatedSpace]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint]		; Get [CurrentStackPoint]
		sub ecx, 16
		mov dword ptr [(StackRegistrationStructure ptr [eax]).CurrentStackPoint], ecx		; Set [CurrentStackPoint]
		movdqu xmm0, oword ptr [DataToPush]
		movdqu oword ptr [ebx + ecx], xmm0													; Push value into the stack
		pop ecx
		pop ebx
		ret
	Push128				endp
	
	Pop8				proc	StructureAddress:DWORD
		push ebx
		push ecx
		push edx
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace]			; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint]		; Get [CurrentStackPoint]
		mov al, byte ptr [ecx + edx]														; Pop and return via AL
		add edx, 1
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], edx		; Set [CurrentStackPoint]
		pop edx
		pop ecx
		pop ebx
		ret
	Pop8				endp
	
	Pop16				proc	StructureAddress:DWORD
		push ebx
		push ecx
		push edx
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace]			; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint]		; Get [CurrentStackPoint]
		mov ax, word ptr [ecx + edx]														; Pop and return via AX
		add edx, 2
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], edx		; Set [CurrentStackPoint]
		pop edx
		pop ecx
		pop ebx
		ret
	Pop16				endp
	
	Pop32				proc	StructureAddress:DWORD
		push ebx
		push ecx
		push edx
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace]			; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint]		; Get [CurrentStackPoint]
		mov eax, dword ptr [ecx + edx]														; Pop and return via EAX
		add edx, 4
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], edx		; Set [CurrentStackPoint]
		pop edx
		pop ecx
		pop ebx
		ret
	Pop32				endp
	
	Pop64				proc	StructureAddress:DWORD										; Not supported in C
		push ebx
		push ecx
		push edx
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace]			; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint]		; Get [CurrentStackPoint]
		movq mm0, qword ptr [ecx + edx]														; Pop and return via MM0
		add edx, 8
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], edx		; Set [CurrentStackPoint]
		pop edx
		pop ecx
		pop ebx
		ret
	Pop64				endp
	
	Pop128				proc	StructureAddress:DWORD										; Not supported in C
		push ebx
		push ecx
		push edx
		mov ebx, dword ptr [StructureAddress]
		mov ecx, dword ptr [(StackRegistrationStructure ptr [ebx]).AllocatedSpace]			; Get [AllocatedSpace]
		mov edx, dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint]		; Get [CurrentStackPoint]
		movdqu xmm0, oword ptr [ecx + edx]													; Pop and return via XMM0
		add edx, 16
		mov dword ptr [(StackRegistrationStructure ptr [ebx]).CurrentStackPoint], edx		; Set [CurrentStackPoint]
		pop edx
		pop ecx
		pop ebx
		ret
	Pop128				endp
	
end